from advanced_alchemy.repository import SQLAlchemyAsyncRepository
from advanced_alchemy.service import SQLAlchemyAsyncRepositoryService

from .models import UserModel


class AuthService(SQLAlchemyAsyncRepositoryService):
    class Repository(SQLAlchemyAsyncRepository[UserModel]):
        model_type = UserModel

    repository_type = Repository


def provide_auth_service(db_session) -> AuthService:
    return AuthService(session=db_session)



from typing import Annotated

from advanced_alchemy.extensions.litestar.dto import SQLAlchemyDTO, SQLAlchemyDTOConfig
from advanced_alchemy.extensions.litestar.providers import create_service_dependencies
from advanced_alchemy.filters import FilterTypes
from advanced_alchemy.service import OffsetPagination
from litestar import Router, get
from litestar.controller import Controller
from litestar.params import Dependency
from msgspec import Struct

from .models import UserModel
from .services import AuthService


class User(Struct):
    name: str


class UserDTO(SQLAlchemyDTO[UserModel]):
    config = SQLAlchemyDTOConfig()


class UserController(Controller):
    dependencies = create_service_dependencies(
        AuthService,
        key="auth_service",
        filters={
            "created_at": True,
            "updated_at": True,
            "sort_field": "email",
            "search": "email",
        },
    )
    return_dto = UserDTO

    @get(operation_id="ListUsers", path="/users")
    async def list_users(
        self,
        auth_service: AuthService,
        filters: Annotated[list[FilterTypes], Dependency(skip_validation=True)],
    ) -> OffsetPagination[UserModel]:
        results, total = await auth_service.list_and_count(*filters)
        return auth_service.to_schema(data=results, total=total, filters=filters)


auth_router = Router(
    path="/api/auth",
    route_handlers=[UserController],
)



import click
from click import Group
from litestar.plugins import CLIPlugin

from ..core.cli import coro
from ..core.database import session_maker
from .factories import UserFactory
from .services import provide_auth_service


class AuthPlugin(CLIPlugin):
    def on_cli_init(self, cli: Group) -> None:
        @cli.group(help="Manage auth, load data with ``load`` command")
        @click.version_option(prog_name="auth")
        def auth(): ...

        @auth.command(help="load auth data")
        @coro
        async def load():
            async with session_maker() as session:
                click.echo("Loading auth data...")

                for _ in range(10):
                    session.add(UserFactory.build())
                await session.commit()

        @auth.command(help="load auth data")
        @coro
        async def reset():
            async with session_maker() as session:
                auth_service = provide_auth_service(session)
                await auth_service.delete_where()
                await auth_service.create_many(
                    [UserFactory.build() for _ in range(10)], auto_commit=True
                )



from src.auth.factories import UserFactory
from src.auth.services import provide_auth_service


async def test_get_users(db_session, test_client):
    auth_service = provide_auth_service(db_session)
    await auth_service.create_many(
        [UserFactory.build() for _ in range(10)], auto_commit=True
    )

    response_json = (await test_client.get("/api/auth/users")).json()
    assert response_json["total"] == 10



import asyncio
from collections.abc import AsyncIterator
from typing import Any, AsyncGenerator

from advanced_alchemy.base import UUIDAuditBase
from litestar import Litestar
from litestar.testing import AsyncTestClient
import pytest
import sqlalchemy
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine

from src.auth import auth_router
from src.core.database import sqlalchemy_plugin
from src.core.settings import settings


@pytest.fixture(scope="session")
def application():
    application = Litestar(
        route_handlers=[auth_router],
        plugins=[sqlalchemy_plugin],
    )
    return application


@pytest.fixture(scope="session")
def event_loop():
    return asyncio.get_event_loop()


@pytest.fixture(scope="function")
async def test_client(application) -> AsyncIterator[AsyncTestClient[Litestar]]:
    async with AsyncTestClient(app=application) as client:
        yield client


@pytest.fixture(scope="session")
async def db():
    yield "ok"
    print("clear")


@pytest.fixture(scope="session")
async def db_session() -> AsyncGenerator[AsyncSession, Any]:
    engine = create_async_engine(url=settings.db_url)

    metadata = UUIDAuditBase.registry.metadata
    async with engine.begin() as conn:
        await conn.run_sync(metadata.drop_all)
        await conn.run_sync(metadata.create_all)

    # create async engine
    async with async_sessionmaker(engine, expire_on_commit=False)() as session:
        try:
            yield session
        except Exception as e:
            await session.rollback()
            raise e
        finally:
            await session.close()

    # clear database
    async with engine.begin() as conn:
        meta = sqlalchemy.MetaData()
        await conn.run_sync(meta.reflect)
        await conn.run_sync(meta.drop_all)



from pydantic import Field
from pydantic_settings import BaseSettings


class Settings(BaseSettings):
    db_url: str = Field(default="postgresql+asyncpg://user:password@localhost:5432/db")


settings = Settings()



from faker import Faker
from polyfactory.factories.sqlalchemy_factory import SQLAlchemyFactory
from polyfactory.fields import Use

from .models import UserModel

faker = Faker()



class UserFactory(SQLAlchemyFactory[UserModel]):
    email = Use(faker.email)
